@@ -39,6 +39,35 @@ static void test_hexstring(const void *test_data)
 	l_free(hex);
 }
 
+static void test_hexstringv(const void *test_data)
+{
+	unsigned char test1[] = { 0x74, 0x65, 0x73, 0x74, 0x00 };
+	unsigned char test2[] = { 0x74, 0x65, 0x73 };
+	struct iovec iov[2];
+	char *hex;
+
+	iov[0].iov_base = test1;
+	iov[0].iov_len = 5;
+	iov[1].iov_base = test2;
+	iov[1].iov_len = 3;
+
+	hex = l_util_hexstringv(iov, 2);
+	assert(hex);
+	assert(!strcmp(hex, "7465737400746573"));
+	l_free(hex);
+}
+
+static void test_hexstring_upper(const void *test_data)
+{
+	unsigned char test[] = { 0x0a, 0x0b, 0x0c, 0xde, 0xf2 };
+	char *hex;
+
+	hex = l_util_hexstring_upper(test, sizeof(test));
+	assert(hex);
+	assert(!strcmp(hex, "0A0B0CDEF2"));
+	l_free(hex);
+}
+
 static void test_from_hexstring(const void *test_data)
 {
 	const char *test = "7465737400";
@@ -113,16 +142,41 @@ static void test_strlcpy(const void *test_data)
 	do_strlcpy(10, 12);
 }
 
+static void test_in_set(const void *test_data)
+{
+	char *a1 = "a";
+	const char *a2 = a1;
+
+	assert(L_IN_SET(1, 1, 2, 3));
+	assert(L_IN_SET(2U, 1, 2, 3));
+	assert(L_IN_SET(3LL, 1, 2, 3));
+	assert(!L_IN_SET(4, 1, 2, 3));
+	assert(!L_IN_SET(4));
+
+	assert(L_IN_STRSET(a1, a2, "b"));
+	assert(L_IN_STRSET(a2, a1, "b"));
+	assert(L_IN_STRSET("b", "a", "b"));
+	assert(!L_IN_STRSET("c", "a", "b"));
+	assert(L_IN_STRSET(NULL, "a", NULL));
+	assert(!L_IN_STRSET(NULL, "a", "b"));
+	assert(!L_IN_STRSET("a", NULL, NULL));
+	assert(!L_IN_STRSET("a"));
+}
+
 int main(int argc, char *argv[])
 {
 	l_test_init(&argc, &argv);
 
 	l_test_add("l_util_hexstring", test_hexstring, NULL);
+	l_test_add("l_util_hexstring_upper", test_hexstring_upper, NULL);
+	l_test_add("l_util_hexstringv", test_hexstringv, NULL);
 	l_test_add("l_util_from_hexstring", test_from_hexstring, NULL);
 
 	l_test_add("l_util_has_suffix", test_has_suffix, NULL);
 
 	l_test_add("l_strlcpy", test_strlcpy, NULL);
 
+	l_test_add("L_IN_SET", test_in_set, NULL);
+
 	return l_test_run();
 }
